;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Phanto, adapted by yoshicookiezeus
;;
;; For usage with Romi's Spritetool
;; Uses extra bit: YES
;; If the extra bit is set, the sprite will only hurt Mario when he's carrying a key.
;;   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

			!RAM_SpriteSpeedY	= $AA
			!RAM_SpriteSpeedX	= $B6
			!RAM_SpriteState	= $C2
			!RAM_SpriteDir		= $157C
			!RAM_SprOAMIndex	= $15EA

			!RAM_ExtraBits		= $7FAB10
			
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

			print "INIT ",pc
			RTL
      

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

			print "MAIN ",pc	;\ you should recognize this part
			PHB			; |
			PHK			; |
			PLB			; |
			JSR SpriteMain		; |
			PLB			; |
			RTL      		;/


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; main
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
AccelerationX:		db $02,$FD
MaxSpeedX:		db $30,$C0
AccelerationY:		db $01,$01
MaxSpeedY:		db $18,$18
			
SpriteMain:		JSR SpriteGraphics
			LDA $14C8,x		;\ if sprite state not normal
			CMP #$08		; |
			BNE Return		;/ return
			LDA $9D			;\ if sprites locked,
			BNE Return		;/ return
			LDA $14c1 : CMP #$0B : BNE Died
			LDA $D8,x : CMP #$80 : BMI Died
			LDA #$00 : STA $14C8,x
			LDA #$01 : STA $1DF9
			Died:
			BRA Chase
			RTS
			
			JSR SubHasKey		;\ if Mario has a key,
			CPY #$01		; |
			BEQ Chase		;/ go to main movement routine

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Secondary movement routine
;
; This routine is executed if Mario isn't holding a key, and will slow Phanto down
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 

			LDA !RAM_ExtraBits,x
			AND #$04
			BNE DontHurtMario

			JSL $01A7DC		; interact with Mario

DontHurtMario:		LDA $14			;\ only change speeds every second frame
			AND #$01		; |
			BNE ApplySpeed		;/

			LDA !RAM_SpriteSpeedX,x	;\ if x speed zero, 
			BEQ XSpeedZero		;/ branch
			BPL XSpeedPositive	; if positive, branch
			INC A
			INC A
XSpeedPositive:		DEC A
			STA !RAM_SpriteSpeedX,x

XSpeedZero:		LDA !RAM_SpriteSpeedY,x	;\ if y speed zero,
			BEQ YSpeedZero		;/ branch
			BPL YSpeedPositive	; if positive, branch
			INC A
			INC A
YSpeedPositive:		DEC A
			STA !RAM_SpriteSpeedY,x
YSpeedZero:		BRA ApplySpeed
Return:			RTS		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Main movement routine
;
; If Mario is holding a key, this routine moves Phanto towards him, and accelerates it
; if the speed cap hasn't been hit
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 

Chase:			JSL $01A7DC		; interact with Mario

			LDA $14			;\ only change speeds every fourth frame
			AND #$03		; |
			BNE ApplySpeed		;/

			JSR SubHorzPos		;\ if max horizontal speed in the appropriate
			LDA !RAM_SpriteSpeedX,x	; | direction achieved,
			CMP MaxSpeedX,y		; |
			BEQ MaxXSpeedReached	;/ don't change horizontal speed
			CLC			;\ else,
			ADC AccelerationX,y	; | accelerate in appropriate direction
			STA !RAM_SpriteSpeedX,x	;/

MaxXSpeedReached:	JSR SubVertPos		;\ if max vertical speed in the appropriate
			LDA !RAM_SpriteSpeedY,x	; | direction achieved,
			CMP MaxSpeedY,y		; |
			BEQ ApplySpeed		;/ don't change vertical speed
			CLC			;\ else,
			ADC AccelerationY,y	; | accelerate in appropriate direction
			STA !RAM_SpriteSpeedY,x	;/

ApplySpeed:		JSL $018022		; apply x speed
			JSL $01801A		; apply y speed
			JSL $019138

	         

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_HAS_KEY
;
; If Mario is holding a key, the Y register is set to 1 after this routine has been run
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;                    

SubHasKey:		PHX			; push Phanto sprite number
			LDY #$00		; clear Y register
		
.HasKey			INY			; increase Y register to show that key is being carried

.Return			PLX			; get Phanto sprite number back
			RTS			; and return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GENERIC GRAPHICS ROUTINE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!Tilemap = $88

SpriteGraphics:		JSR GetDrawInfo	

			LDA $00			; set x position of the tile
			STA $0300,y

			LDA $01			; set y position of the tile
			STA $0301,y

			LDA #!Tilemap
			STA $0302,y

			LDA $15F6,x		; get sprite palette info
			ORA $64			; add in the priority bits from the level settings
			STA $0303,y		; set properties

			LDY #$02		; the tiles drawn were 16x16
			LDA #$00		; one tile was drawn
			JSL $01B7B3		; finish OAM write
			RTS
			
			
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_HORZ_POS
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B817
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SubHorzPos:		LDY #$00
			LDA $D1
			SEC
			SBC $E4,x
			STA $0F
			LDA $D2
			SBC $14E0,x
			BPL .Return
			INY
.Return			RTS



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_VERT_POS
; This routine determines if Mario is above or below the sprite.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B829
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SubVertPos:		LDY #$00
			LDA $D3
			CLC
			ADC #$10
			SEC
			SBC $D8,x
			STA $0F
			LDA $D4
			SBC $14D4,x
			BPL .Return
			INY
.Return			RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;org $03B75C

Table1:			db $0C,$1C
Table2:			db $01,$02

GetDrawInfo:		STZ $186C,x		; reset sprite offscreen flag, vertical
			STZ $15A0,x		; reset sprite offscreen flag, horizontal
			LDA $E4,x		; \
			CMP $1A			;  | set horizontal offscreen if necessary
			LDA $14E0,x		;  |
			SBC $1B			;  |
			BEQ .OnScreenX		;  |
			INC $15A0,x		; /

.OnScreenX		LDA $14E0,x		; \
			XBA			;  |
			LDA $E4,x		;  |
			REP #$20		;  |
			SEC			;  |
			SBC $1A			;  | mark sprite invalid if far enough off screen
			CLC			;  |
			ADC #$0040		;  |
			CMP #$0180		;  |
			SEP #$20		;  |
			ROL A			;  |
			AND #$01		;  |
			STA $15C4,x		; / 
			BNE .Invalid		; 
			
			LDY #$00		; \ set up loop:
			LDA $1662,x		;  | 
			AND #$20		;  | if not smushed (1662 & 0x20), go through loop twice
			BEQ .OnScreenLoop	;  | else, go through loop once
			INY			; / 
.OnScreenLoop		LDA $D8,x		; \ 
			CLC			;  | set vertical offscreen if necessary
			ADC Table1,y		;  |
			PHP			;  |
			CMP $1C			;  | (vert screen boundry)
			ROL $00			;  |
			PLP			;  |
			LDA $14D4,x		;  | 
			ADC #$00		;  |
			LSR $00			;  |
			SBC $1D			;  |
			BEQ .OnScreenY		;  |
			LDA $186C,x		;  | (vert offscreen)
			ORA Table2,y		;  |
			STA $186C,x		;  |
.OnScreenY		DEY			;  |
			BPL .OnScreenLoop	; /

			LDY $15EA,x		; get offset to sprite OAM
			LDA $E4,x		; \ 
			SEC			;  | 
			SBC $1A			;  | $00 = sprite x position relative to screen boarder
			STA $00			; / 
			LDA $D8,x		; \ 
			SEC			;  | 
			SBC $1C			;  | $01 = sprite y position relative to screen boarder
			STA $01			; / 
			RTS			; return

.Invalid		PLA			; \ return from *main gfx routine* subroutine...
			PLA			;  |    ...(not just this subroutine)
			RTS			; /